@@ -4,23 +4,9 @@ import numpy as np
 from io import StringIO
 import pylab as plt
 from dask.distributed import as_completed
+from py4xs.utils import run
 
-def run(cmd, path="", ignoreErrors=True, returnError=False, debug=False):
-    """ cmd should be a list, e.g. ["ls", "-lh"]
-        path is for the cmd, not the same as cwd
-    """
-    cmd[0] = path+cmd[0]
-    p = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
-    out, err = p.communicate()
-    if debug:
-        print(out.decode(), err.decode())
-    if len(err)>0 and not ignoreErrors:
-        print(err.decode())
-        raise Exception(err.decode())
-    if returnError:
-        return out.decode(),err.decode()
-    else:
-        return out.decode()
+atsas_path = ""
     
 def extract_vals(txt, dtype=float, strip=None, debug=False):
     if strip is not None:
@@ -40,7 +26,7 @@ def atsas_create_temp_file(fn, d1s, skip=0, q_cutoff=0.6):
     idx = (d1s.qgrid<=q_cutoff)
     np.savetxt(fn, np.vstack([d1s.qgrid[idx][skip:], d1s.data[idx][skip:], d1s.err[idx][skip:]]).T)
     
-def atsas_autorg(fn, debug=False, path=""):
+def atsas_autorg(fn, debug=False, path=atsas_path):
     ret = run(["autorg", fn], path).split('\n')
     #rg,drg = extract_vals(ret[0], "+/-", debug=debug)
     #i0,di0 = extract_vals(ret[1], "+/-", debug=debug)
@@ -63,7 +49,7 @@ def atsas_autorg(fn, debug=False, path=""):
             "fit range": [n1,n2],
             "quality": qual}
 
-def atsas_datgnom(fn, rg, first, last=None, fn_out=None, path=""):
+def atsas_datgnom(fn, rg, first, last=None, fn_out=None, path=atsas_path):
     """ 
     """
     if fn_out is None:
@@ -95,10 +81,10 @@ def atsas_datgnom(fn, rg, first, last=None, fn_out=None, path=""):
             #    Real space range:                    0.0000 to      45.9000
             #    Real space Rg:                   0.1470E+02 +-   0.1148E+00
             #    Real space I(0):                 0.5733E+01 +-   0.2619E-01
-            qual = extract_vals(run(["grep", "Total Estimate", fn_out], path))[0]
-            dmax = extract_vals(run(["grep", "Real space range", fn_out], path))[1]
-            rgg = extract_vals(run(["grep", "Reciprocal space Rg", fn_out], path))[0]
-            rgp = extract_vals(run(["grep", "Real space Rg", fn_out], path))[0]
+            qual = extract_vals(run(["grep", "Total Estimate", fn_out]))[0]
+            dmax = extract_vals(run(["grep", "Real space range", fn_out]))[1]
+            rgg = extract_vals(run(["grep", "Reciprocal space Rg", fn_out]))[0]
+            rgp = extract_vals(run(["grep", "Real space Rg", fn_out]))[0]
     except:
         qual = 0
         dmax = 100
@@ -122,10 +108,11 @@ def read_arr_from_strings(lines, cols=[0,1,2]):
         tb = np.genfromtxt(StringIO(buf))
         if np.isnan(tb).any():   # mixed text and numbersS          J EXP       ERROR       J REG       I REG
             continue
-        ret.append([tb[i] for i in cols])
+        if len(tb)>=np.max(cols):
+            ret.append([tb[i] for i in cols])
     return np.asarray(ret).T
 
-def read_gnom_out_file(fn, plot_pr=False, ax=None):
+def read_gnom_out_file(fn, plot_data=False, fig=None, figsize=(9,4)):
     ff = open(fn, "r")
     tt = ff.read()
     ff.close()
@@ -135,15 +122,22 @@ def read_gnom_out_file(fn, plot_pr=False, ax=None):
     iq, pr = t1.split("####      Real Space Data                               ####")
     #iq, pr = t1.split("Distance distribution  function of particle")
     dq, di = read_arr_from_strings(iq.rstrip().split("\n"), cols=[0,-1])
+    rdq,rdi,rde,_ = read_arr_from_strings(iq.rstrip().split("\n"), cols=[0,1,2,3])
     dr, dpr, dpre = read_arr_from_strings(pr.rstrip().split("\n"), cols=[0,1,2])
     
-    if plot_pr:
-        if ax is None:
-            plt.figure()
-            ax = plt.gca()
-        ax.errorbar(dr, dpr, dpre)
+    if plot_data:
+        if fig is None:
+            fig = plt.figure(figsize=figsize)
+        ax1 = fig.add_subplot(121)
+        ax1.semilogy(dq, di)
+        ax1.errorbar(rdq, rdi, rde)
+        ax2 = fig.add_subplot(122)
+        ax2.errorbar(dr, dpr, dpre)
     
-    return hdr.rstrip(),dq,di,dr,dpr,dpre
+    return {"header": hdr.rstrip(),
+            "raw_data": [rdq,rdi,rde],
+            "gnom_fit": [dq,di],
+            "P(r)": [dr,dpr,dpre]}
 
 # ALMERGE   Automatically merges data collected from two different concentrations or 
 #           extrapolates it to infinite dilution assuming moderate particle interactions.
@@ -248,7 +242,7 @@ Mandatory arguments to long options are mandatory for short options too.
 
 """
 
-def atsas_dat_tools(fn_out, path=""):
+def atsas_dat_tools(fn_out, path=atsas_path):
     # datporod: the used Rg, I0, the computed volume estimate and the input file name
     #
     # datvc: the first three numbers are the integrated intensities up to 0.2, 0.25 and 0.3, respectively. 
@@ -284,7 +278,7 @@ def atsas_dat_tools(fn_out, path=""):
 
 
 def gen_atsas_report(d1s, ax=None, fig=None, sn=None, skip=0, q_cutoff=0.6, 
-                     plot_full_q_range=False, print_results=True, path=""):
+                     plot_full_q_range=False, print_results=True, path=atsas_path):
     if not os.path.isdir("processed"):
         os.mkdir("processed")
     
@@ -329,7 +323,10 @@ def gen_atsas_report(d1s, ax=None, fig=None, sn=None, skip=0, q_cutoff=0.6,
     re_gnom = atsas_datgnom(tfn, re_autorg["Rg"], first=sk0+1,
                             last=len(d1s.qgrid[d1s.qgrid<=qc0]), fn_out=tfn_out, path=path)
     try:
-        hdr,dq,di,dr,dpr,dpre = read_gnom_out_file(tfn_out)
+        ret = read_gnom_out_file(tfn_out)
+        hdr = ret["header"]
+        dq,di = ret["gnom_fit"]
+        dr,dpr,dpre = ret["P(r)"]
         if plot_full_q_range:
             idx = (dq>=d1s.qgrid[0])
             ax[0].loglog(dq[idx], di[idx], zorder=2)
@@ -418,7 +415,7 @@ def gen_atsas_report(d1s, ax=None, fig=None, sn=None, skip=0, q_cutoff=0.6,
         txt += f"MW estimate: {ret['datmow']['MW']/1000:.1f} kDa (MoW)"          
         return txt
     
-def gen_report(fn):
+def gen_report(fn, atsas_path=''):
     """ create a pdf file to summarize the static solution scattering data in the specified h5 file
     """
     dn = os.path.dirname(fn)
@@ -445,6 +442,7 @@ def gen_report(fn):
     with open(fn1, 'r+') as fh:
         txt = fh.read()
         txt = re.sub('00template00.h5', fn, txt)
+        txt = re.sub('00atsas_path00', atsas_path, txt)            
         fh.seek(0)
         fh.write(txt)
         fh.truncate()
