@@ -19,8 +19,10 @@ def getUID():
         return "-".join(qstr.split("-")[:-1])
     return qstr
 
-def makeQRxls(fn, n=10):
+def makeQRxls(fn, n=10, use_UID_text=True):
     qdict = {"UIDs" : [getUID() for _ in  range(n)] }
+    if use_UID_text:
+        qdict['label'] = [uid[0:4] for uid in qdict['UIDs']]
     df = pd.DataFrame.from_dict(qdict)
     df.to_excel(fn, index=False)
 
@@ -95,8 +97,9 @@ def validate_sample_list(xls_fn,
     if sh_name is None:
         sh_name = xl.sheet_names[0]
     msg.append(f"reading data from {sh_name} ...") 
-    df = xl.parse(sh_name, dtype={'Volume (uL)': float})    
-
+    df = xl.parse(sh_name)
+    df['Volume (uL)'] = pd.to_numeric(df['Volume (uL)'], errors='coerce')
+    
     # template spreadsheet is protected, "lix template"
     if check_template:
         sdict = df.to_dict()['Notes']
@@ -230,10 +233,9 @@ def process_sample_lists(xls_fns, process_all_tabs=False, b_lim=4):
             elif bdict['proposal_id']!=bdict0['proposal_id'] or bdict['proposal_id']!=bdict0['proposal_id']:
                 raise Exception("Cannot process plates from differernt proposal/SAF together.")
             pdict[sh_name] = [wdict, mdict]
-    
-    # get ot2 layout info
-    
+        
     # create tranfer list
+    mixing_list = []
     transfer_list = []
     slist = []
     for bcode in pdict.keys():
@@ -258,32 +260,37 @@ def process_sample_lists(xls_fns, process_all_tabs=False, b_lim=4):
             else:
                 vt = wdict[wn]["Volume (uL)"]
             # require mixing
+            blow_out = False
             if wn in mdict.keys():
                 # mix first
                 for ws,v in mdict[wn].items():
-                    transfer_list.append([pname, ws, pname, wn, v])
+                    mixing_list.append([pname, ws, pname, wn, [v, blow_out]])
+                    if not blow_out:
+                        blow_out = True
                     vt += v
-
+                mixing_list[-1][-1][-1] = vt
+                    
             # now transfer
             transfer_list.append([pname, wn, hname, hpos, vt])
             
             # add entry into the sample dictionary
             slist.append([hname, hpos, wdict[wn]["Sample"], wdict[wn]["Buffer"], vt])
 
-    return slist,transfer_list,bdict
+    return slist,transfer_list,bdict,mixing_list
 
-def read_OT2_layout(plate_slots, holder_slots, msg=None):
+def read_OT2_layout(plate_slots, holder_slots, msg=None, ot2_ip="169.254.246.65", ot2_port=22):
     """ 
     the arguments should be a comma-separated list of slot positions on the Opentron deck
     e.g. "1,2"  
+    revise IP and port when using ssh tunnel to connect to OT2
     """
     if msg is None:
         ssh_key = str(pathlib.Path.home())+"/.ssh/ot2_ssh_key"
         if not os.path.isfile(ssh_key):
             raise Exception(f"{ssh_key} does not exist!")
 
-        cmd = ["ssh", "-i", ssh_key, "-o", "port=9999",
-               "root@localhost", "/var/lib/jupyter/notebooks/check_deck_config.py", 
+        cmd = ["ssh", "-i", ssh_key, "-o", f"port={ot2_port}",
+               f"root@{ot2_ip}", "/var/lib/jupyter/notebooks/check_deck_config.py", 
                "-h", holder_slots, "-p", plate_slots]
 
         ret = subprocess.run(cmd, capture_output=True)
@@ -329,11 +336,12 @@ def generate_measurement_spreadsheet(fn, slist, holders, bdict):
 def generate_docs(ot2_layout, xls_fns, ldict=None,   
                   run_name="test",
                   b_lim=4,
-                  plate_type = "corning_96_wellplate_360ul_flat",
+                  plate_type = "biorad_96_wellplate_200ul_pcr",
                   holder_type = "lix_3x_holder_c",
                   tip_type = "opentrons_96_tiprack_300ul",
-                  flow_rate_aspirate = 50, flow_rate_dispense = 50, 
-                  bottom_clearance = 1
+                  flow_rate_aspirate = 20, flow_rate_dispense = 50, 
+                  n_mix=3, pause_after_mixing=True,
+                  bottom_clearance = 0.5
                  ):
     """ ot2_layout should be a dictionary:
             {"plates" : "1,2",
@@ -341,7 +349,7 @@ def generate_docs(ot2_layout, xls_fns, ldict=None,
              "tips" : "9,10"}
     """
     print("Processing sample list(s) ...")
-    slist,transfer_list,bdict = process_sample_lists(xls_fns, b_lim=b_lim)
+    slist,transfer_list,bdict,mixing_list = process_sample_lists(xls_fns, b_lim=b_lim)
     
     if ldict is None:
         print("Reading bar/QR codes, this might take a while ...")
@@ -366,7 +374,7 @@ def generate_docs(ot2_layout, xls_fns, ldict=None,
     protocol = ["metadata = {'protocolName': 'sample transfer',\n",
                 "            'author': 'LiX',\n",
                 "            'description': 'auto-generated',\n",
-                "            'apiLevel': '2.3'\n",
+                "            'apiLevel': '2.9'\n",
                 "           }\n", 
                 "\n",
                 "def run(ctx):\n",]
@@ -384,7 +392,16 @@ def generate_docs(ot2_layout, xls_fns, ldict=None,
     protocol.append(f"    pipet.flow_rate.aspirate = {flow_rate_aspirate}\n")
     protocol.append(f"    pipet.flow_rate.dispense = {flow_rate_dispense}\n")
 
-    for st in transfer_list:
+    if pause_after_mixing:
+        ops_list = mixing_list+["pause"]+transfer_list
+    else:
+        ops_list = mixing_list+transfer_list
+    
+    bl_str = "blow_out=True, blowout_location='destination well'"
+    for st in ops_list:
+        if st=="pause":
+            protocol.append(f"    ctx.pause('Mixing completed. Resume to start transfer.')\n")
+            continue
         src,sw,dest,dw,vol = st
         if dest in holders.keys():
             dest = holders[dest]
@@ -401,7 +418,18 @@ def generate_docs(ot2_layout, xls_fns, ldict=None,
         else:
             raise Exception(f"Unknown labware encountered: {dest}")
 
-        protocol.append(f"    pipet.transfer({vol}, {sname}, {dname})\n")
+        if isinstance(vol, list):
+            # part of mixing
+            vol,action = vol  
+            #protocol.append(f"    pipet.transfer({vol}, {sname}, {dname})\n")
+            #if action==True:
+            #    protocol.append(f"    pipet.blow_out()\n")
+            if action>1: # mixing 
+                protocol.append(f"    pipet.transfer({vol}, {sname}, {dname}, mix_after=({n_mix}, {action}), {bl_str})\n")
+            else:
+                protocol.append(f"    pipet.transfer({vol}, {sname}, {dname}, {bl_str})\n")
+        else:
+            protocol.append(f"    pipet.transfer({vol}, {sname}, {dname}, {bl_str})\n")
 
     fd = open(fn, "w+")
     fd.writelines(protocol)
@@ -460,7 +488,7 @@ def generate_docs2(ot2_layout, xls_fns,
     protocol = ["metadata = {'protocolName': 'sample transfer',\n",
                 "            'author': 'LiX',\n",
                 "            'description': 'auto-generated',\n",
-                "            'apiLevel': '2.3'\n",
+                "            'apiLevel': '2.9'\n",
                 "           }\n", 
                 "\n",
                 "def run(ctx):\n",]
